﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Optimization;
using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.OptimizationExpertSystem.Common {
  public class KNearestNeighborModel : IRecommendationModel {
    private readonly int K;
    private readonly string[] characteristics;

    public KNearestNeighborModel(int k, string[] characteristics) {
      this.K = k;
      this.characteristics = characteristics;
    }

    public IEnumerable<Tuple<IAlgorithm, double>> GetRanking(KnowledgeCenter kc) {
      var distances = kc.GetProblemDistances(characteristics);
      var performances = new Dictionary<IAlgorithm, List<double>>();
      for (var k = 0; k < K; k++) {
        if (distances.Count == 0) break;
        var min = distances.MinItems(x => x.Value).First();
        // lookup algorithm performances in min
        var perfs = kc.GetAlgorithmPerformance(min.Key);
        if (perfs.Count == 0) {
          k--;
          continue;
        }
        foreach (var p in perfs) {
          var ert = p.Value;
          if (double.IsNaN(ert)) ert = int.MaxValue;
          List<double> erts;
          if (!performances.TryGetValue(p.Key, out erts)) {
            performances[p.Key] = new List<double>() { ert }; ;
          } else erts.Add(ert);
        }
        distances.Remove(min.Key);
      }

      return performances.Select(x => new { Alg = x.Key, Perf = x.Value.Average() })
                         .OrderBy(x => x.Perf)
                         .Select(x => Tuple.Create(x.Alg, x.Perf));
    }
  }
}
