﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common.Resources;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.MainForm;
using HeuristicLab.OptimizationExpertSystem.Common;
using HeuristicLab.PluginInfrastructure;
using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.OptimizationExpertSystem {
  [View("Performance Modeling View")]
  [Content(typeof(KnowledgeCenter), IsDefaultView = false)]
  public partial class PerformanceModelingView : KnowledgeCenterViewBase {
    private readonly CheckedItemList<StringValue> characteristics;

    public PerformanceModelingView() {
      InitializeComponent();
      characteristics = new CheckedItemList<StringValue>();
      characteristics.CheckedItemsChanged += CharacteristicsOnCheckedItemsChanged;
      characteristicsViewHost.Content = characteristics;
      recommendStartButton.Text = string.Empty;
      recommendStartButton.Image = VSImageLibrary.Play;
      refreshCharacteristicsButton.Text = string.Empty;
      refreshCharacteristicsButton.Image = VSImageLibrary.Refresh;
      UpdateRecommenderCombobox();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      UpdateCharacteristics();
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      // TODO: up/download state
      recommendStartButton.Enabled = Content != null && recommenderComboBox.SelectedIndex >= 0 && characteristics.CheckedItems.Any();
      characteristicsViewHost.Enabled = Content != null;
    }

    #region Update Controls
    private void UpdateRecommenderCombobox() {
      if (InvokeRequired) { Invoke((Action)UpdateRecommenderCombobox); return; }
      var prevSelection = (IAlgorithmInstanceRecommender)recommenderComboBox.SelectedItem;
      var prevNewIndex = -1;
      recommenderComboBox.Items.Clear();

      var i = 0;
      foreach (var rcmd in ApplicationManager.Manager.GetInstances<IAlgorithmInstanceRecommender>()) {
        recommenderComboBox.Items.Add(rcmd);
        if (prevSelection == null || rcmd.GetType() == prevSelection.GetType())
          prevNewIndex = prevSelection == null ? 0 : i;
        i++;
      }
      
      recommenderComboBox.SelectedIndex = prevNewIndex;
    }

    private void UpdateCharacteristics() {
      if (InvokeRequired) { Invoke((Action)UpdateCharacteristics); return; }

      var @checked = new HashSet<string>(characteristics.CheckedItems.Select(x => x.Value.Value));
      characteristics.Clear();
      if (Content == null || Content.ProblemInstances.Count == 0) return;

      foreach (var c in Content.ProblemInstances.ResultNames) {
        characteristics.Add(new StringValue(c), @checked.Contains(c));
      }
    }
    #endregion

    #region Content Event Handlers
    protected override void OnProblemInstancesChanged() {
      base.OnProblemInstancesChanged();
      UpdateCharacteristics();
      SetEnabledStateOfControls();
    }
    #endregion

    #region Control Event Handlers
    private void RecommenderComboBoxOnSelectedIndexChanged(object sender, EventArgs e) {
      if (InvokeRequired) { Invoke((Action<object, EventArgs>)RecommenderComboBoxOnSelectedIndexChanged, sender, e); return; }
      var rcmd = (IAlgorithmInstanceRecommender)recommenderComboBox.SelectedItem;
      parameterCollectionView.Content = rcmd != null ? rcmd.Parameters : null;
      SetEnabledStateOfControls();
    }

    private void RecommendStartButtonOnClick(object sender, EventArgs e) {
      var rcmd = (IAlgorithmInstanceRecommender)recommenderComboBox.SelectedItem;
      Content.RecommendationModel = rcmd.TrainModel(Content, characteristics.CheckedItems.Select(x => x.Value.Value).ToArray());
    }

    private void RefreshCharacteristicsButtonOnClick(object sender, EventArgs e) {
      UpdateCharacteristics();
      SetEnabledStateOfControls();
    }
    #endregion

    #region Other Event Handlers
    private void CharacteristicsOnCheckedItemsChanged(object sender, EventArgs e) {
      SetEnabledStateOfControls();
    }
    #endregion
  }
}
